<?php

if (!defined(DS))
    define('DS',DIRECTORY_SEPARATOR);

load_lib('file');

class account_documents extends Module {

    /**
     * Таблица корневых папок пользователей.
     */
    static $tableUserFolder = 'account_document_user_folders';

    /*
     * Таблица папок пользователя
     */
    static $tableFolders = 'account_document_folders';

    /*
     * Таблица файлов пользователя
     */
    static $tableFiles = 'account_document_files';

    protected $root_folder = 'content/users/';

    protected $user_id = null;

    protected $user_folder = null;

    protected $user_folder_id = null;

    protected $user_login = null;

    /**
     * Не загружаем файлы:
     */
    public static $ignore_file = array('php','exe','sh','cgi','bat');

    /**
     * НЕ создаем/сканируем папки:
     */
    public static $ignore_folders = array('.','..');

    function __construct($user_id){
        parent::__construct();

        $this->set_id($user_id);

        $folder = $this->db->query('SELECT `dir_name`,`user_folder_id` FROM `'.self::$tableUserFolder.'` WHERE `user_id`=:id',array('id'=>$this->user_id),db::FETCH_ROW);
        if (!empty($folder)){
            $this->user_folder      =$folder['dir_name'];   
            $this->user_folder_id   =$folder['user_folder_id']; 
        }
    }

    public function load(){}
    /**
     * Если требуется устанавливаем id пользователя
     * @var int $user_id 
     */
    public function set_id($user_id){
        $this->user_id = $user_id;
    }

    /**
     * Создаем папку для пользователя
     */
    public function create(){
        if ($dirname = $this->create_user_root_folder($this->user_id)){
                $this->db->insert(self::$tableUserFolder, array(
                    'user_id'=>$this->user_id,
                    'dir_name'=>$dirname,
            ));
            return $dirname;
        }
        else return false;
    }

    /**
     * Создаем физическую папку пользователя.
     */
    protected function create_user_root_folder(){
        $account = new account();
        if (!$account->id_exists($this->user_id))
            return false;
        $login = $account->get_login($this->user_id);
        $folder_name = uniqid($login.'_');
        if (!is_dir($this->root_folder.$folder_name) && is_writable($this->root_folder)){
            //что бы поставить папке права 0777 нужно выставить umask на 0
            $old = umask(0);
                mkdir($this->root_folder.$folder_name,0777);
            umask($old);
            return $folder_name;
        }
        else return false;
    }

    /**
     * Создаем виртуальную папку
     * @var string $folder_name - имя папки
     */
    public function create_folder($folder_name){
        $alias  = $folder_name;
        $name   = file_translit($folder_name);
        $name   = NormalizeFolderName($name);
        $dir    = $this->get_user_folder();
        $this->db->insert(self::$tableFolders,array(
                'user_folder_id'=> $this->user_folder_id,
                'alias'         => $alias,
                'name'          => $name,
            ));
    }

    /**
     * Удаляем папку
     * @var string $folder_id - имя папки
     * Создаем папку
     */
    public function delete_folder($folder_id){
        $this->db->delete(self::$tableFolders,array('user_folder_id'=>$this->user_folder_id, 'folder_id'=>$folder_id));
    }

    /**
     * Получть имя родительской папки
     */
    public function get_user_folder($autocreate = false){
        $folder_name = $this->user_folder;
        if ($autocreate === true && !$folder_name){
            $folder_name = $this->create();
        }
        if (!$folder_name) return false;
        $this->check_user_folder($folder_name);
        $this->user_folder = $folder_name;
        return $folder_name;
    }
    /**
     * Проверка на существования папки. если нет, создаем.
     */
    private function check_user_folder($folder_name){
        if(!is_dir($this->root_folder.$folder_name) && is_writable($this->root_folder)){
        //что бы поставить папке права 0777 нужно выставить umask на 0
        $old = umask(0);
            mkdir($this->root_folder.$folder_name,0777);
        umask($old);
        }
    }

    public function get_full_folder(){
        return CR.$this->root_folder.$this->get_user_folder();
    }

    /**
     * Получить папку по id
     * @param $folder_id
     */
    public function get_folder_by_id($folder_id){
        return $this->db->query('SELECT * FROM `'.self::$tableFolders.'` WHERE `folder_id`=:id',array('id'=>$folder_id),db::FETCH_ROW);
    }

    /**
     * Получить содержимое папки
     * @var id $folder_id - id папки
     */
    function get_items($folder_id){
        $items = $this->db->query('SELECT * FROM `'.self::$tableFiles.'` WHERE `folder_id`=:id',array('id'=>$folder_id),db::FETCH_ALL);
        $out = array();
        foreach($items as $item){
            $out[$item['file_id']] = array(
                'id' => $item['file_id'],
                'folder_id' => $item['folder_id'],
                'alias' => $item['alias'],
                'name' => $item['name'],
                'size' => $item['size'],
                'full_path' => SR.$this->root_folder.$this->user_folder.DS.$item['name'],
                );
        }
            return $out;

    }

    /**
     * Получаем имена папок. если parent_folder_id пустой, возвращаем папки из корня //реализовать
     * @var string $folder_name  - имя папки
     */
    public function get_folders($parent_folder_id= null){
        if ($this->user_folder_id){
        $folders = $this->db->query('SELECT * FROM `'.self::$tableFolders.'` WHERE `user_folder_id`=:folder_id',array('folder_id'=>$this->user_folder_id),db::FETCH_ALL);

        $items = array();
        if (!empty($folders)){
        foreach($folders as $item){
            $items[$item['folder_id']]= array(
                    'name'      => $item['alias'],
                    //'path_to' => $this->root_folder.$this->user_folder,
                    'path'      => $item['name'],
                    'folder_id' => $item['folder_id'],
                    );
                }
                return $items;
            }
        }
            return false;
    }



    public function get_file($file_id){
        return $file = $this->db->query('SELECT * FROM `'.self::$tableFiles.'` WHERE `file_id`=:id',array('id'=>$file_id),db::FETCH_ROW);
    }

    public function download_file($file_id){
        $file = $this->get_file($file_id);
        $file_ext = explode('.',$file['name']);
        if (count($file_ext)>1)
            $file_ext = end($file_ext);
        $filename = array($file['alias'],$file_ext);
        //зафигачили имя
        $filename = implode('.',$filename);
        //отдаем все
        header('Content-type: application/zhopa');
        header('Content-Disposition: attachment; filename="'.$filename.'"');
        echo file_get_contents(SR.$this->root_folder.$this->user_folder.DS.$file['name']);
        die();
    }

    public function delete_file($file_id){
        $dir = $this->get_user_folder();
        $file = $this->get_file($file_id);
        if (!empty($file)){
            $this->db->delete(self::$tableFiles,array('file_id'=>$file_id));
            unlink (SR.$this->root_folder.$this->user_folder.DS.$file['name']);
        }
    }

    /**
     * Загрузка файлов в папку
     * @var string $path - путь куда загружать файл
     */

    public function upload_file($folder_id){
        if (is_uploaded_file($_FILES['file']['tmp_name'])) {
            $ext = explode('.',$_FILES['file']['name']);
            $ext = end($ext);
                if (in_array($ext,self::$ignore_file))
                    return false;

            $filename = explode('.',$_FILES['file']['name']);
            //выкидываем расширение
            if (count($filename)>1)
                    array_pop($filename);
            $insert['alias'] = implode('.',$filename);

            //реальное имя
                $curr = $this->get_folder_by_id($folder_id);
                $name = $curr['name'].'_'.file_translit($_FILES['file']['name']);
            $insert['name'] = $name;
            $insert['size'] = $_FILES['file']['size'];
            $insert['folder_id'] = $folder_id;
            

            if (move_uploaded_file($_FILES['file']['tmp_name'],SR.$this->root_folder.$this->user_folder.DS.$name)){
                $this->db->insert(self::$tableFiles,$insert);
            }
        }
    }
}
?>