<?php

/**
 *  :
 *  - development,  
 *  - production,    
 *  - auto,         - 
 */
if (!defined("MENOCOM_ENV"))
    define("MENOCOM_ENV", isset($_SERVER['MENOCOM_ENV']) ? $_SERVER['MENOCOM_ENV'] : "auto");

/**
 *       
 *      ,   
 *   
 * 
 * @since 07.07.2011
 * @author  'Alurin' 
 */
class Config implements ArrayAccess, IteratorAggregate, Countable {
    protected static $_instance = null;
    protected $_options = array();
    protected $_readonly = false;

    protected function _load($filename) {
        $fullname = DR . "configs/{$filename}.php";
        if (file_exists($fullname)) {
            $config = $this;
            require_once $fullname;
        }
    }

    /**
     *       
     * @author  'Alurin' 
     */
    public static function load() {
        if (self::$_instance == null) {
            self::$_instance = new config();
            self::$_instance->_load("global");
            self::$_instance->_load(strtolower(MENOCOM_ENV));
        }
        return self::$_instance;
    }

    public function __construct(array $options = array()) {
        foreach ($options as $offset => $value) {
            $this->__set($offset, $value);
        }
    }

    public function __get($offset) {
        return $this->offsetGet($offset);
    }

    public function __set($offset, $value) {
        return $this->offsetSet($offset, $value);
    }
    
    public function count() {
        return count($this->_options);
    }

    public function assign($offset, $value = null) {
        if (is_array($offset)) {
            foreach ($offset as $offset => $value) {
                $this->offsetSet($offset, $value);
            }
        } else {
            $this->offsetSet($offset, $value);
        }
        return $this;
    }

    public function get($offset, $default) {
        $value = $this->offsetGet($offset);
        return $value === null ? $default : $value;
    }

    public function set($offset, $value) {
        return $this->offsetSet($offset, $value);
    }

    public function offsetExists($offset) {
        return isset($this->_options[$offset]);
    }

    public function offsetGet($offset) {
        if (isset($this->_options[$offset])) {
            return $this->_options[$offset];
        }

        return null;
    }

    public function offsetSet($offset, $value) {
        if ($this->_readonly) {
            throw new Exception('Config is read only');
        }
        if ($value === null) {
            unset($this->_options[$offset]);
            return;
        }
        if (isset($this->_options[$offset]) && $this->_options[$offset] instanceof self) {
            $this->_options[$offset]->assign((array) $value);
        } else {
             if (is_array($value)) {
                $value = new self($value);
            }
            $this->_options[$offset] = $value;
        }
        return $this;
    }

    public function offsetUnset($offset) {
        unset($this->_options[$offset]);
        return $this;
    }
    
    public function toArray() {
        $result = array();
        foreach ($this->_options as $key => $value) {
            if ($value instanceof self) {
                $value = $value->toArray();
            }
            $result[$key] = $value;
        }
        return $result;
    }
    public function getIterator() {
        return new ArrayIterator($this->_options);
    }
}
