<?php

load_lib('utily');

class menocms_generator {
    const ACTION_CLEAR  = 1;
    const ACTION_CREATE = 2;
    const ACTION_ALL    = 3;
    /**
     *
     * @var Db
     */
    protected $db = null;

    protected $_generators = array();

    protected $_root = array();

    protected $_data = array();

    protected $_uniques = array();

    public static $table_columns = array();

    public function __construct(array $data) {
        $this->db = App::get()->db;
        $this->_data = $data;
    }

    //       
    public function add_data($name, array $data) {
        if (isset($this->_data[$name])) {
            $this->_data[$name] = array_merge($this->_data[$name], $data);
        } else {
            $this->_data[$name] = $data;
        }
        return $this;
    }

    //     
    public function generate($name, $has_null = false) {
        if (is_string($name)) {
            $values = $this->_data[$name];
        } else {
            $values = (array) $name;
        }
        $value = null;
        if (count($values) > 0) {
            while ($value == null) {
                if ($has_null) {
                    $id = rand(0, count($values)) - 1;
                } else {
                    $id = rand(1, count($values)) - 1;
                }
                if (isset($values[$id])) {
                    $value = $values[$id];
                }
                if ($value == null && $has_null) {
                    continue;
                }
            }
        } 
        return $value;
    }

    //    
    public function generate_number($lenght) {
        $value = '';
        for ($i = 0; $i < $lenght; $i++) {
            $value .= rand(0, 9);
        }
        return $value;
    }

    function generate_unique(array $parts, $group = '', $translit = false) {
        if ($translit)
            $parts = array_map('translitIt', $parts);
        $prefix = trim(implode('_', array_filter($parts)), '_');
        $string = $prefix;
        if (!isset($this->_uniques[$group])) {
            $this->_uniques[$group] = array();
        }
        while ($string == null || in_array($string, $this->_uniques[$group])) {
            $string = $prefix . rand(0, 1000000);
        }
        $this->_uniques[$group][] = $string;
        return $translit ? translitIt($string) : $string;
    }
    
    function generate_string($unique = true, array $parts = array(), $translit = false) {
        if ($unique === true) {
            $string = $this->generate_unique($parts, '__strings', $translit);
        } elseif ($unique != false) {
            $string = $this->generate_unique($parts, $unique, $translit);
        } else {
            $string = 'STRING???';
        }
        return $string;
    }

    /**
     *  
     * @param type $unique  bool   -   ?
     * @param type $parts   string - ,    
     * @return string
     */
    public function generate_email($unique = false, array $parts = array()) {
        if ($unique === true) {
            $prefix = $this->generate_unique($parts, '__emails', true);
        } elseif ($unique != false) {
            $prefix = $this->generate_unique($parts, $unique, true);
        } else {
            $prefix = rand(0, 1000000);
        }
        return strtolower($prefix . '@example.com');
    }

    public function generate_date() {
        return date("Y-m-d H:i:s", (mktime() - rand(0, 60*60*365*42)));
    }
    
    /**
     *   
     * @param type $name
     * @param type $title    
     * @param array $params   :
     *                      nethod          - string -              - create_$name ( create_)
     *                      table           - string -                                - $name
     *                      count           - int    -                           - 150
     *                      min_count       - int    -                     - 0
     *                      random_count    - bool   -    min_count  count    - false
     *                      required        - bool   -    -         - false
     */
    function add_generator($name, $title, array $params = array()) {
        $unifid = str_replace(array('-', '.'), '_', $name);
        $this->_generators[$name] = array_merge(
            array(
                'method'            => 'create_' . $unifid,
                'table'             => $unifid,
                'random_count'      => false,
                'count'             => 150,
                'min_count'         => 0,
                'required'          => false
            ),
            $params, array(
                'title'             => $title,
                'children'          => array(),
                'cicles_count'      => 0,
                'last_id'           => 0
            ));

        $this->_root[] = $name;
        if (isset($params['parent'])) {
            $this->add_children($params['parent'], $name);
        }
    }

    function add_children($parent, $name) {
        if (in_array($name, $this->_root) && isset($this->_generators[$parent])) {
            unset($this->_root[ array_search($name, $this->_root) ]);
            $this->_generators[$parent]['children'][] = $name;
        }
    }

    function get_last_id($name) {
        if (isset($this->_generators[$name]['last_id'])) {
            return $this->_generators[$name]['last_id'];
        }
        return null;
    }
    
    //  
    private function _trancateTable($id) {
        $options = $this->_generators[$id];
        foreach ($options['children'] as $child) {
            $this->_trancateTable($child);
        }
        $this->db->query('TRUNCATE TABLE :table', array('table' => db_name($options['table'])));
    }

    // 
    private function _insertData($table, array $values) {
        $values = $this->db->get_request($table, $values);
        $this->db->insert($table, $values);
        return $this->db->get_id();
    }

    private function _create($name = null, $id = null, array $data = array()) {
        if ($name == null) {
            $generators = $this->_root;
        } elseif (isset($this->_generators[$name])) {
            $generators = $this->_generators[$name]['children'];
        } else {
            $generators = array();
        }
        
        foreach ($generators as $name) {
            if (isset($this->_generators[$name])) {
                $options = $this->_generators[$name];

                //     
                $method = $options['method'];
                $count  = ($options['random_count'] ? rand($options['min_count'], $options['count']) : $options['count']);

                //   
                for ($i = 0; $i < $count; $i++) {
                    $gen_data = $this->{$method}($id, $data);
                    if (is_array($gen_data)) {
                        $gen_id   = $this->_insertData($options['table'], $gen_data);
                        //    
                        $this->_create($name, $gen_id, $gen_data);
                        $this->_generators[$name]['cicles_count']++;
                        $this->_generators[$name]['last_id'] = $gen_id;
                    } else {
                        if ($options['required']) {
                            $i--;
                        }
                    }
                }
            }
        }
    }

    ///      
    function execute($action = self::ACTION_ALL) {
        echo '<ul>';
        if ($action == self::ACTION_ALL || $action == self::ACTION_CLEAR) {
            echo '<li> </li>';
            foreach ($this->_root as $name) {
                $this->_trancateTable($name);
            }
        }
        
        if ($action == self::ACTION_ALL || $action == self::ACTION_CREATE) {
            echo '<li>   </li>';
            $this->_create();
            echo '<ul>';
            foreach ($this->_generators as $options) {
                echo '<li><strong>' . $options['title'] . '</strong>: ' . $options['cicles_count'] . '</li>';
            }
            echo '</ul>';
        }
        echo '</ul>';
    }
}