<?php

abstract class Module {

    public static $table_columns = array();
    /**
     *
     * @var Db
     */
    protected $db = null;
    /**
     *
     * @var int|null
     */
    public $id = null;
    public $data = array();
    public $current = array();

    public function __construct(Db $db = null, $id = null) {
        if ($db == null) {
            $db = App::get()->db;
        }

        $this->db = $db;
        $this->id = $id;
        // if ($db) $this->load();
    }
    public function setId($id){
        $this->id = $id;
    }

    public function _stripslashes($value) {
        if (is_string($value))
            return stripslashes($value);
        return $value;
    }

    public function get_request_fill( array $values = null) {
        if ($values === null) {
            $values = $_REQUEST;
        }
        if (get_magic_quotes_gpc())
            $values = array_map(array($this, '_stripslashes'), $values);
        return $values;
    }

    public function get_request($table_name, array $values = null) {
        $values = $this->get_request_fill($values);

        if (!isset(self::$table_columns[$table_name])) {
            $result = $this->db->query("DESCRIBE `" . $table_name . '`');
            while ($row = mysql_fetch_assoc($result)) {
                self::$table_columns[$table_name][] = $row['Field'];
            }
        }
        $result = array();
        foreach (self::$table_columns[$table_name] as $field) {
            if (isset($values[$field]))
                $result[$field] = $values[$field];
        }
        return $result;
    }

    public abstract function load();
    
    protected function _load_all($table_name) {
        return $this->db->query("SELECT * FROM `" . $table_name . '`', null, DB::FETCH_ALL);
    }

    protected function _load($table_name, $id, $column_id = "id") {
        return $this->db->query("SELECT * FROM `" . $table_name . '` WHERE ' . $column_id . ' = :id', array(
            "id" => $id
                ), DB::FETCH_ROW);
    }

    protected function _load_page($table_name, $page, $item_per_page) {
        $full_count = $this->db->query('SELECT COUNT(*) FROM `' . $table_name . '`', null, Db::FETCH_RESULT);
        if (!$full_count)
            return null;
        $page_count = ceil($full_count / $item_per_page);

        if ($page > $page_count) {
            $page = $page_count;
        }

        $data = $this->db->query('SELECT * FROM `' . $table_name . '` LIMIT :page, :count', array(
                    "page" => ($page - 1) * $item_per_page,
                    "count" => $item_per_page
                        ), DB::FETCH_ALL);

        return array(
            "count" => $full_count,
            "item_per_page" => $item_per_page,
            "first" => $page * $item_per_page,
            "last" => $page * $item_per_page + count($data),
            "data" => $data
        );
    }

    protected function _create($table_name, array $options = null) {
        $options = $this->get_request($table_name, $options);
        $this->db->insert($table_name, $options);
        $this->id = $this->db->get_id();
    }

    protected function _insert($table_name, array $options = null) {
        $options = $this->get_request($table_name, $options);
        $this->db->insert($table_name, $options);
        $this->id = $this->db->get_id();
    }


    protected function _update($table_name, $id, array $options = null) {
        if ($id == null)
            return;

        $options = $this->get_request($table_name, $options);
        // var_dump($options,$_REQUEST);die();
        $this->db->update($table_name, $options, array(
            "id" => $id
        ));
    }

    protected function _delete($table_name, $id, $column_id = "id") {
        if ($id == null)
            return;

        $options = $this->get_request($table_name, $options);
        $this->db->delete($table_name, array(
            $column_id => $id
        ));
    }
}

?>