<?php

class Navigation {

    protected static $instance;
    protected $active_items = array();
    protected $breadcrumb_separator = '>';
    protected $breadcrumbs = array();
    protected $site_name;

    private function __constructor() {
        
    }

    /**
     * @return Navigation
     */
    protected static function get_instance() {
        if (self::$instance == null)
            self::$instance = new self();
        return self::$instance;
    }

    ///
    public static function get_active_items() {
        $nav = self::get_instance();
        return $nav->active_items;
    }
    
    ///   
    public static function add_active_item($module, $id = null) {
        $id = ($id != 0) ? $id : null;
        $nav = self::get_instance();
        if ($id == null) {
            $nav->active_items[$module]['active'] = true;
        } else {
            $nav->active_items[$module]['active'] = true;
            $nav->active_items[$module][$id] = true;
        }
    }

    ///    
    public static function add_deactive_item($module, $id = null) {
        $id = ($id != 0) ? $id : null;
        $nav = self::get_instance();
        if ($id == null) {
            $nav->active_items[$module]['active'] = false;
        } else {
            $nav->active_items[$module][$id] = false;
        }
    }

    ///        
    public static function set_deactive_module($module) {
        $nav = self::get_instance();
        unset($nav->active_items[$module]);
    }

    ///  ?
    public static function is_active_item($module, $id = null) {
        $id = ($id != 0) ? $id : null;
        $nav = self::get_instance();
        
        if ($id == null)
            return isset($nav->active_items[$module]['active']) && $nav->active_items[$module]['active'];
        else
            return isset($nav->active_items[$module][$id]) && $nav->active_items[$module][$id];
    }

    ///   
    public static function set_site_name($name) {
        $nav = self::get_instance();
        $nav->site_name = $name;
    }

    public static function unshift_breadcrumb($title, $href = null) {
        $nav = self::get_instance();
        array_unshift($nav->breadcrumbs, array(
            'title' => $title,
            'href' => $href
        ));
    }

    ///     
    public static function add_breadcrumb($title, $href = null) {
        if ($href) $href = '/'.trim($href, '/');
        $nav = self::get_instance();
        $nav->breadcrumbs[] = array(
            'title' => $title,
            'href' => $href
        );
    }

    ///     
    public static function get_breadcrumbs($status_string) {
        //    
        $status_string = trim($status_string);
        if (!empty($status_string)) {
            $segments = explode(" / ", $status_string);
            $matches = array();
            //preg_match('#[ ]*[/-]?[ ]*(<a.*href=["\'](?P<href>.*)["\'].*>(?P<title>.*)</a>|(?P<title2>[^<].*))*#si', $status_string, $matches);
            //var_dump($matches);
            foreach ($segments as $page) {
                $page = trim($page, '/');
                preg_match('#[ ]*[/-]?[ ]*(<a.*href=["\'](?P<href>.*)["\'].*>(?P<title>.*)</a>|(?P<title2>[^<].*))*#si', $page, $matches);
                if (!empty($matches['title2'])) {
                    self::add_breadcrumb($matches['title2']);
                } elseif (!empty($matches[0]['title'])) {
                    self::add_breadcrumb($matches['title'], @$matches['href']);
                }
            }
        }
        ///
        $nav = self::get_instance();
        $status_string = '';
        $request_uri = '/' . trim($_SERVER['REQUEST_URI'], '/');
		
		$last = end(array_keys($nav->breadcrumbs));
        foreach ($nav->breadcrumbs as $key => $bread) {
            if($last==$key) break;
            // var_dump($bread['href']);
            if ($bread['href'] == null || $bread['href'] == $request_uri) {
                $status_seg = '<span>'.$bread['title'].'</span>';
            } else {
                $status_seg = '<a href="' . $bread['href'] . ( $key!= 0 && trim($bread['href'],'/') ? '/' : '') . '">' . $bread['title'] . '</a>';
            }
            $status_string .= ( $status_string == null ? '' : ' ' . $nav->breadcrumb_separator . ' ') . $status_seg;
        }
        return $status_string;
    }

    // Title: [ ] | [] | [...] | [] -  
    // (    )

    public static function get_title_metatag($title,$del_first = true) {
        $nav = self::get_instance();
        $title = trim($title);
        if (empty($title)) {
            array_shift($nav->breadcrumbs);
            foreach (array_reverse($nav->breadcrumbs) as $param) {
                    $title .= ( $title == null ? '' : ' | ') . $param['title'];
            }
        }
        $title = strip_tags(trim($title));
        return strip_tags(($title ? $title . '  ' : '') . strip_tags($nav->site_name));
    }

    // Descriptions: [ ] - [ ]
    public static function get_description_metatag($description) {
            $nav = self::get_instance();
        $description = trim($description);
        if (empty($description)) {
            $temp = $nav->breadcrumbs;
            $page = array_pop($temp);
            if ($page != null) {
                $description = $page['title'];
            }
            $description .=  '  ' . $nav->site_name;
        }
        $description = str_replace('"', '\'', $description);
        return ($description ? $description : '');
    }

    // Keywords: [ ], [], [...], [] -  
    public static function get_keywords_metatag($keywords) {
        $nav = self::get_instance();
        $keywords = trim($keywords);
        if (empty($keywords)) {
            foreach (array_reverse($nav->breadcrumbs) as $param) {
                $keywords .= ( $keywords == null ? '' : ', ') . $param['title'];
            }
            $keywords .= '  ' . $nav->site_name;
            $keywords = str_replace('"', '\'', $keywords);
        }
        return ($keywords ? $keywords  : '') ;
    }

}

?>